%% User prompt video
clear;
clc;
done = false;

% Setting up Maps
map = containers.Map('KeyType', 'char', 'ValueType', 'any');
% number to keep track of each athlete's data so they don't overwrite each
% other
athleteNum = 1;

cameraNum = input('How many cameras are being used? ');

%% While loop for indoor track when there are 2 cameras
while (done == false && cameraNum == 2)

    % Print if there are no current athletes
    if isempty(map)
        fprintf('No athletes currently \n');
    else
        % Print all athletes currently in the system
        % Meant to show names so ppl know how ppl are written in the system and
        % won't record people twice
        fprintf('Current athletes: \n');
        disp(map.keys);
    end

    % Prompt to see who's video they are trying to input
    name = input('Name of athlete in video: ', 's');

    % Check to see if this person already has a struct in the map
    if (isKey(map, name))
        % if the person already has data, should set the variables that are
        % being written to so they correspond to their data
        tempMap = map(name);
        jumpNum = tempMap.jumpNum + 1; %increase jumpNum
        jumpDataS = tempMap.jumpDataS; %accessing previous jumpData
        jumpDataF = tempMap.jumpDataF;
        
    else
        %if the person does not have data, need to create a new struct to store
        %their data
        % Creating struct to store info abt athlete in one place
        %jumpNum for athlete
        %jumpData to store data for all jumps
        athleteData(athleteNum) = struct('jumpNum',1, 'jumpDataS', struct([]), 'jumpDataF', struct([]));

        %still need to set variables to be written to later in the code
        jumpNum = 1;

        %putting athlete into map
        map(name) = athleteData(athleteNum);
        athleteNum = athleteNum + 1; %add 1 to athleteNum so data isn't overwritten
    end

    file_sagittal = input('Add sagittal video file: ', 's');
    file_frontal = input('Add frontal video file: ', 's');

    videoLabeler(file_sagittal) %https://www.mathworks.com/help/vision/ug/get-started-with-the-video-labeler.html
    %https://www.mathworks.com/matlabcentral/answers/128697-using-image-processing-tool-measure-the-angle
    % Identify hip, knee, and ankle tracker
    % Automate markers using point tracker
    % export data

    videoLabeler(file_frontal)
    % identify right and left hip, knee, and ankle markers
    % Exported Labels
    
    while 1
        complete = input('Done labeling? (Y/N)', 's');
            if(complete == 'Y')
                break;
            else
                continue;
            end
    end

    
    gTruthS %display label properties

    gTruthS.LabelDefinitions %display labels

    labelDataS = gTruthS.LabelData; %store label data
    head(labelDataS) %display label data

    %Creates struct with position data over time range
    %Label data is in format [x y width height]
    %y axis goes in reverse

    %For loop to only get data that has info
    started = false;
    for i = 1:height(labelDataS)
        %if there is no data
        if isempty(labelDataS(i,:).hip{1})
            %if this is the first instance of no data, set end time to then
            if started == true
                endTime = labelDataS(i,:).Time;
                break;
            else
                continue;

            end
        %if there is data
        else
            %if this is the first instance of data, set start time to then
            if started == false
                startTime = labelDataS(i,:).Time;
                started = true;
            else
                continue;
            end
        end
    end

    totalTime = datenum((startTime-endTime)*86400);

    gTruthIntervalS = labelDataS(timerange(startTime,endTime),:);
    % should prob change time range to full range of data
    head(gTruthIntervalS)

    %gTruthInterval(frame,:).label{1}(1=x,2=y)
    %gets y-value in the first frame

    %% While loop - sagittal analysis

    % Setting up struct to store jump data
    jumpDataS(jumpNum) = struct('time', [], 'hipx', [], 'hipy', [], 'kneex', [], 'kneey', [], ...
    'anklex', [], 'ankley' , [], 'hipdx', [], 'hipdy', [], ...
    'hipd', [], 'hipavgv', [],'hipavgv_vertical',[], 'flexion', [], 'angularv', []);

    %Setting up arrays to store values
    time = [];
    hipx = [];
    hipy = [];
    kneex = [];
    kneey = [];
    anklex = [];
    ankley = [];
    hipdx = [];
    hipdy = [];
    hipd = [];
    hipavgv = [];
    hipavgv_vertical = [];
    flexion = [];
    angularv = [];

    i=height(gTruthIntervalS);
    x=1;
    while x<=i
        hipx(x)= gTruthIntervalS(x,:).hip{1}(1);
        hipy(x)= gTruthIntervalS(x,:).hip{1}(2);
        kneex(x)= gTruthIntervalS(x,:).knee{1}(1);
        kneey(x)= gTruthIntervalS(x,:).knee{1}(2);
        anklex(x)= gTruthIntervalS(x,:).ankle{1}(1);
        ankley(x)= gTruthIntervalS(x,:).ankle{1}(2);

        P0 = [gTruthIntervalS(x,:).knee{1}(1), gTruthIntervalS(x,:).knee{1}(2)]; %defining each point
        P1 = [gTruthIntervalS(x,:).ankle{1}(1), gTruthIntervalS(x,:).ankle{1}(2)];
        P2 = [gTruthIntervalS(x,:).hip{1}(1), gTruthIntervalS(x,:).hip{1}(2)];

        n1 = (P2 - P0) / norm(P2 - P0);  % Normalized vectors
        n2 = (P1 - P0) / norm(P1 - P0);
        angle = atan2(norm(det([n2; n1])), dot(n1, n2));
        flexion(x) = rad2deg(angle);
        time(x) = datenum((gTruthIntervalS(x,:).Time)*86400);

        if x>=4
            hipdx(x) = hipx(x) - hipx(x-3);
            hipdy(x) = hipy(x) - hipy(x-3);
            hipd(x) = sqrt(hipdx(x)^2 + hipdy(x)^2);
            hipavgv(x) = hipd(x) / 0.1;
            hipavgv_vertical(x) = hipdy(x)/0.1;
            angularv(x) = (flexion(x) - flexion(x-3))/0.1;
        else
            hipdx(x) = 0;
            hipdy(x) = 0;
            hipd(x) = 0;
            hipavgv(x) = 0;
            hipavgv_vertical(x) = 0;
            angularv(x) = 0;
        end

        timePassed = time(x) - datenum((startTime)*86400);
        if timePassed > totalTime/2
            if hipavgv_vertical(x)>157.48 
                disp('High hip average vertical velocity detected \n')
            else
            end
            if angularv(x)<-8000
                disp('High average angular velocity detected \n')
            else
            end
        else
        end
        
        x = x+1;
    end

    jumpDataS(jumpNum).time=time';
    jumpDataS(jumpNum).hipx=hipx';
    jumpDataS(jumpNum).hipy=hipy';
    jumpDataS(jumpNum).kneex=kneex';
    jumpDataS(jumpNum).kneey=kneey';
    jumpDataS(jumpNum).anklex=anklex';
    jumpDataS(jumpNum).ankley=ankley';
    jumpDataS(jumpNum).flexion=flexion';
    jumpDataS(jumpNum).hipdx=hipdx';
    jumpDataS(jumpNum).hipdy=hipdy';
    jumpDataS(jumpNum).hipd=hipd';
    jumpDataS(jumpNum).hipavgv=hipavgv';
    jumpDataS(jumpNum).hipavgv_vertical = hipavgv_vertical';
    jumpDataS(jumpNum).angularv=angularv';

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataS(i).time(1);
        plot(jumpDataS(i).time-startTime, jumpDataS(i).hipavgv, 'o')
        hold on
    end
    title("Hip Average Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Hip Velocity')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataS(i).time(1);
        plot(jumpDataS(i).time-startTime, jumpDataS(i).hipavgv_vertical, 'o')
        hold on
    end
    title("Hip Average Vertical Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Hip Vertical Velocity')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataS(i).time(1);
        plot(jumpDataS(i).time-startTime, jumpDataS(i).hipavgv, 'o')
        hold on
    end
    title("Knee Flexion Angle: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Knee Flexion Angle')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataS(i).time(1);
        plot(jumpDataS(i).time-startTime, jumpDataS(i).angularv, 'o')
        hold on
    end
    title("Knee Angular Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Knee Angular Velocity')
    hold off

    %Write sagittal data to spreadhseet

    sheetName = append(name, string(jumpNum));
    S=[jumpDataS(jumpNum)];
    writetable(struct2table(S),'Lsagittal.xlsx', 'Sheet', sheetName);
    winopen Lsagittal.xlsx

    %% Frontal Analysis

    gTruthF %display label properties

    gTruthF.LabelDefinitions %display labels

    labelDataF = gTruthF.LabelData; %store label data
    head(labelDataF) %display label data

    %For loop to only get data that has info
    started = false;
    for i = 1:height(labelDataF)
        %if there is no data
        if isempty(labelDataF(i,:).kneer{1})
            %if this is the first instance of no data, set end time to then
            if started == true
                endTime = labelDataF(i,:).Time;
                break;
            else
                continue;

            end
        %if there is data
        else
            %if this is the first instance of data, set start time to then
            if started == false
                startTime = labelDataF(i,:).Time;
                started = true;
            else
                continue;
            end
        end
    end

    gTruthIntervalF = labelDataF(timerange(startTime,endTime),:);
    % should prob change time range to full range of data
    head(gTruthIntervalF)

    % Setting up struct to store jump data
    jumpDataF(jumpNum) = struct('time', [], 'hipx_r', [], 'hipx_l', [], 'kneex_r', [], 'kneex_l', [], ...
    'anklex_r', [], 'anklex_l' , [], 'valgus', []);

    % Boolean to keep track of if there is valgus positioning in both legs
    isValgus = false;

    j = height(gTruthIntervalF); %value for indexing
    x=1;

    % Boolean to keep track of if there are any more hip values or not
    hipr_stopped = false;
    hipl_stopped = false;

    % Setting up arrays to store values
    time = [];
    hipx_r = [];
    hipx_l = [];
    valgus = [];
    kneex_r = [];
    kneex_l = [];
    anklex_r = [];
    anklex_l = [];

    while x<=j
        
        %Checking to see if there is marker data for the hips since
        %sometimes the markers do not track the full landing
        if isempty(gTruthIntervalF(x,:).hipr{1})
            if hipr_stopped == false
                last_hipxr = hipx_r(x-1);
            else
            end
            hipx_r(x) = last_hipxr;
        else
            hipx_r(x)= gTruthIntervalF(x,:).hipr{1}(1);
        end

        % Checking left hip and setting value
        if isempty(gTruthIntervalF(x,:).hipl{1})
            if hipl_stopped == false
                last_hipxl = hipx_l(x-1);
            else
            end
            hipx_l(x) = last_hipxl;
        else
            hipx_l(x)= gTruthIntervalF(x,:).hipl{1}(1);
        end


        hipx_r(x)= gTruthIntervalF(x,:).hipr{1}(1);
        hipx_l(x)= gTruthIntervalF(x,:).hipl{1}(1);
        kneex_r(x)= gTruthIntervalF(x,:).kneer{1}(1);
        kneex_l(x)= gTruthIntervalF(x,:).kneel{1}(1);
        anklex_r(x)= gTruthIntervalF(x,:).ankler{1}(1);
        anklex_l(x)= gTruthIntervalF(x,:).anklel{1}(1);
        time(x) = datenum((gTruthIntervalF(x,:).Time)*86400);

        %Checking if there is valgus positioning on right
        if kneex_r(x)<hipx_r(x) && kneex_r(x)<anklex_r(x)
            valgus_r(x) = "Valgus";
        else
            valgus_r(x) = "No Valgus";
        end

        %Checking if there is valgus positioning on left
        if kneex_l(x)>hipx_l(x) && kneex_l(x)>anklex_l(x)
            valgus_l(x) = "Valgus";
        else
            valgus_l(x) = "No Valgus";
        end

        if strcmp(valgus_r(x),"Valgus") && strcmp(valgus_l, "Valgus")
            isValgus = true;
            valgus(x) = "Valgus";
        else
            valgus(x) = "";
        end

        x=x+1;
    end

    if isValgus == true
        fprintf('Valgus Positioning Detected \n')
    else
    end

    jumpDataF(jumpNum).time = time';
    jumpDataF(jumpNum).hipx_r = hipx_r';
    jumpDataF(jumpNum).hipx_l = hipx_l';
    jumpDataF(jumpNum).kneex_r = kneex_r';
    jumpDataF(jumpNum).kneex_l = kneex_l';
    jumpDataF(jumpNum).anklex_r = anklex_r';
    jumpDataF(jumpNum).anklex_l = anklex_l';
    jumpDataF(jumpNum).valgus = valgus';


    %Write frontal data to spreadsheet
    sheetName = append(name, string(jumpNum));
    S=[jumpDataF(jumpNum)];
    writetable(struct2table(S),'Frontal.xlsx', 'Sheet', sheetName);
    winopen Frontal.xlsx

    %% Updating and continuing

    % Need to update variables in map
    tempMap = map(name);
    tempMap.jumpDataS = jumpDataS;
    tempMap.jumpDataF = jumpDataF;
    map(name) = tempMap;
    

    while 1
    isDone = input('Add another video? (Y/N)', 's');
    if (isDone == 'Y')
        done = false;
        break
    elseif (isDone == 'N')
            done = true;
            break
    else
        fprintf('Invalid response. Please write Y or N \n');
        continue;
    end
    end
end
    

%% While loop for outdoor track when there are 3 cameras
while (done == false && cameraNum == 3)

    % Print if there are no current athletes
    if isempty(map)
        fprintf('No athletes currently \n');
    else
        % Print all athletes currently in the system
        % Meant to show names so ppl know how ppl are written in the system and
        % won't record people twice
        fprintf('Current athletes: \n');
        disp(map.keys);
    end

    % Prompt to see who's video they are trying to input
    name = input('Name of athlete in video: ', 's');

    % Check to see if this person already has a struct in the map
    if (isKey(map, name))
        % if the person already has data, should set the variables that are
        % being written to so they correspond to their data
        jumpNum = map(name).jumpNum + 1; %increase jumpNum
        jumpDataR = map(name).jumpDataR; %all right sagittal jump data
        jumpDataL = map(name).jumpDataL; %all left sagittal jump data
        jumpDataF = map(name).jumpDataF; %all frontal jump data
    else
        %if the person does not have data, need to create a new struct to store
        %their data
        % Creating struct to store info abt athlete in one place
        %jumpNum for athlete
        %need to think abt min angle and such and if they should be stored
        %separately which they prob should
        athleteData(athleteNum) = struct('jumpNum',1, 'jumpDataR', struct([]), 'jumpDataL', struct([]), 'jumpDataF', struct([]));

        %still need to set variables to be written to later in the code
        jumpNum = 1;


        %putting athlete into map
        map(name) = athleteData(athleteNum);
        athleteNum = athleteNum + 1; %add 1 to athleteNum so data isn't overwritten
    end

    file_sagittal_r = input('Add right sagittal video file: ', 's');
    file_sagittal_l = input('Add left sagittal video file: ', 's');
    file_frontal = input('Add frontal video file: ', 's');

    videoLabeler(file_sagittal) %https://www.mathworks.com/help/vision/ug/get-started-with-the-video-labeler.html
    %https://www.mathworks.com/matlabcentral/answers/128697-using-image-processing-tool-measure-the-angle
    % Identify hip, knee, and ankle tracker
    % Automate markers using point tracker
    % export data

    videoLabeler(file_frontal)
    % identify right and left hip, knee, and ankle markers
    % Exported Labels
    
    while 1
        complete = input('Done labeling? (Y/N)', 's');
            if(complete == 'Y')
                break;
            else
                continue;
            end
    end


%% Handling right sagittal video data
    
    gTruthSR %display label properties

    gTruthSR.LabelDefinitions %display labels

    labelDataSR = gTruthSR.LabelData; %store label data
    head(labelDataSR) %display label data

    %Creates struct with position data over time range
    %Label data is in format [x y width height]
    %y axis goes in reverse

    %For loop to only get data that has info
    started = false;
    for i = 1:height(labelDataSR)
        %if there is no data
        if isempty(labelDataSR(i,:).hip{1})
            %if this is the first instance of no data, set end time to then
            if started == true
                endTime = labelDataSR(i,:).Time;
                break;
            else
                continue;

            end
        %if there is data
        else
            %if this is the first instance of data, set start time to then
            if started == false
                startTime = labelDataSR(i,:).Time;
                started = true;
            else
                continue;
            end
        end
    end

    totalTime = datenum((startTime-endTime)*86400);

    gTruthIntervalSR = labelDataSR(timerange(startTime,endTime),:);
    % should prob change time range to full range of data
    head(gTruthIntervalSR)
    
    % Setting up struct to store jump data
    jumpDataR(jumpNum) = struct('time', [], 'hipx', [], 'hipy', [], 'kneex', [], 'kneey', [], ...
    'anklex', [], 'ankley' , [], 'hipdx', [], 'hipdy', [], ...
    'hipd', [], 'hipavgv', [],'hipavgv_vertical',[], 'flexion', [], 'angularv', []);

    % While loop
    i=height(gTruthIntervalSR);
    x=1;
    while x<=i
        hipx(x)= gTruthIntervalSR(x,:).hip{1}(1);
        hipy(x)= gTruthIntervalSR(x,:).hip{1}(2);
        kneex(x)= gTruthIntervalSR(x,:).knee{1}(1);
        kneey(x)= gTruthIntervalSR(x,:).knee{1}(2);
        anklex(x)= gTruthIntervalSR(x,:).ankle{1}(1);
        ankley(x)= gTruthIntervalSR(x,:).ankle{1}(2);

        P0 = [gTruthIntervalSR(x,:).knee{1}(1), gTruthIntervalSR(x,:).knee{1}(2)]; %defining each point
        P1 = [gTruthIntervalSR(x,:).ankle{1}(1), gTruthIntervalSR(x,:).ankle{1}(2)];
        P2 = [gTruthIntervalSR(x,:).hip{1}(1), gTruthIntervalSR(x,:).hip{1}(2)];

        n1 = (P2 - P0) / norm(P2 - P0);  % Normalized vectors
        n2 = (P1 - P0) / norm(P1 - P0);
        angle = atan2(norm(det([n2; n1])), dot(n1, n2));
        flexion(x) = rad2deg(angle);

        if x>=4
            hipdx(x) = hipx(x) - hipx(x-3);
            hipdy(x) = hipy(x) - hipy(x-3);
            hipd(x) = sqrt(hipdx(x)^2 + hipdy(x)^2);
            hipavgv(x) = hipd(x) / 0.1;
            hipavgv_vertical(x) = hipdy(x)/0.1;
            angularv(x) = (flexion(x) - flexion(x-3))/0.1;
        else
            hipdx(x) = 0;
            hipdy(x) = 0;
            hipd(x) = 0;
            hipavgv(x) = 0;
            hipavgv_vertical(x) = 0;
            angularv(x) = 0;
        end

        if time(x) > totalTime/2
            if hipavgv_vertical(x)>157.48 
                disp('High hip average vertical velocity detected \n')
            else
            end
            if angularv(x)<-8000
                disp('High average angular velocity detected \n')
            else
            end
        else
        end

        x = x+1;
    end

    jumpDataR(jumpNum).hipx=hipx';
    jumpDataR(jumpNum).hipy=hipy';
    jumpDataR(jumpNum).kneex=kneex';
    jumpDataR(jumpNum).kneey=kneey';
    jumpDataR(jumpNum).anklex=anklex';
    jumpDataR(jumpNum).ankley=ankley';
    jumpDataR(jumpNum).flexion=flexion';
    jumpDataR(jumpNum).hipdx=hipdx';
    jumpDataR(jumpNum).hipdy=hipdy';
    jumpDataR(jumpNum).hipd=hipd';
    jumpDataR(jumpNum).hipavgv=hipavgv';
    jumpDataR(jumpNum).hipavgv_vertical=hipavgv_vertical';
    jumpDataR(jumpNum).angularv=angularv';

    
    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum-1
        startTime = jumpDataR(i).time(1);
        plot(jumpDataR(i).time-startTime, jumpDataR(i).hipavgv, 'o')
        hold on
    end
    title("Right Hip Average Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Hip Velocity')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum-1
        startTime = jumpDataR(i).time(1);
        plot(jumpDataR(i).time-startTime, jumpDataR(i).hipavgv_vertical, 'o')
        hold on
    end
    title("Right Hip Average Vertical Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Hip Vertical Velocity')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataR(i).time(1);
        plot(jumpDataR(i).time-startTime, jumpDataR(i).hipavgv, 'o')
        hold on
    end
    title("Right Knee Flexion Angle: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Knee Flexion Angle')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataR(i).time(1);
        plot(jumpDataR(i).time-startTime, jumpDataR(i).angularv, 'o')
        hold on
    end
    title("Right Knee Angular Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Knee Angular Velocity')
    hold off

    %Wrtie sagittal data to spreadhseet
    sheetName = append(name, string(jumpNum));
    S=[jumpDataS(jumpNum, :)];
    writetable(struct2table(S),'Rsagittal.xlsx', 'Sheet', sheetName);
    winopen Rsagittal.xlsx

%% Handling left sagittal video data

    gTruthSL %display label properties

    gTruthSL.LabelDefinitions %display labels

    labelDataSL = gTruthSL.LabelData; %store label data
    head(labelDataSL) %display label data

    %Creates struct with position data over time range
    %Label data is in format [x y width height]
    %y axis goes in reverse

    %For loop to only get data that has info
    started = false;
    for i = 1:height(labelDataSL)
        %if there is no data
        if isempty(labelDataSL(i,:).hip{1})
            %if this is the first instance of no data, set end time to then
            if started == true
                endTime = labelDataSL(i,:).Time;
                break;
            else
                continue;

            end
        %if there is data
        else
            %if this is the first instance of data, set start time to then
            if started == false
                startTime = labelDataSL(i,:).Time;
                started = true;
            else
                continue;
            end
        end
    end

    totalTime = datenum((startTime-endTime)*86400);

    gTruthIntervalSL = labelDataSL(timerange(startTime,endTime),:);
    % should prob change time range to full range of data
    head(gTruthIntervalSL)

    %gTruthInterval(frame,:).label{1}(1=x,2=y)
    %gets y-value in the first frame

    % Setting up struct to store jump data
    jumpDataL(jumpNum) = struct('hipx', [], 'hipy', [], 'kneex', [], 'kneey', [], ...
    'anklex', [], 'ankley' , [], 'hipdx', [], 'hipdy', [], ...
    'hipd', [], 'hipavgv', [], 'hipavgv_vertical', [], 'flexion', [], 'angularv', []);

    % While loop

    i=height(gTruthIntervalSL);
    x=1;
    while x<=i
        hipx(x)= gTruthIntervalSL(x,:).hip{1}(1);
        hipy(x)= gTruthIntervalSL(x,:).hip{1}(2);
        kneex(x)=gTruthIntervalSL(x,:).knee{1}(1);
        kneey(x)= gTruthIntervalSL(x,:).knee{1}(2);
        anklex(x)= gTruthIntervalSL(x,:).ankle{1}(1);
        ankley(x)= gTruthIntervalSL(x,:).ankle{1}(2);

        P0 = [gTruthIntervalSL(x,:).knee{1}(1), gTruthIntervalSL(x,:).knee{1}(2)]; %defining each point
        P1 = [gTruthIntervalSL(x,:).ankle{1}(1), gTruthIntervalSL(x,:).ankle{1}(2)];
        P2 = [gTruthIntervalSL(x,:).hip{1}(1), gTruthIntervalSL(x,:).hip{1}(2)];

        n1 = (P2 - P0) / norm(P2 - P0);  % Normalized vectors
        n2 = (P1 - P0) / norm(P1 - P0);
        angle = atan2(norm(det([n2; n1])), dot(n1, n2));
        flexion(x) = rad2deg(angle);

        if x>=4
            hipdx(x) = hipx(x) - hipx(x-3);
            hipdy(x) = hipy(x) - hipy(x-3);
            hipd(x) = sqrt(hipdx(x)^2 + hipdy(x)^2);
            hipavgv(x) = hipd(x) / 0.1;
            hipavgv_vertical(x) = hipavgv_vertical(x)/0.1;
            angularv(x) = (flexion(x) - flexion(x-3))/0.1;
        else
            hipdx(x) = 0;
            hipdy(x) = 0;
            hipd(x) = 0;
            hipavgv(x) = 0;
            hipavgv_vertical(x) = 0;
            angularv(x) = 0;
        end

        if time(x) > totalTime/2
            if hipavgv_vertical(x)>157.48 
                disp('High hip average vertical velocity detected \n')
            else
            end
            if angularv(x)<-8000
                disp('High average angular velocity detected \n')
            else
            end
        else
        end

        x = x+1;
    end

    jumpDataL(jumpNum).hipx=hipx';
    jumpDataL(jumpNum).hipy=hipy';
    jumpDataL(jumpNum).kneex=kneex';
    jumpDataL(jumpNum).kneey=kneey';
    jumpDataL(jumpNum).anklex=anklex';
    jumpDataL(jumpNum).ankley=ankley';
    jumpDataL(jumpNum).flexion=flexion';
    jumpDataL(jumpNum).hipdx=hipdx';
    jumpDataL(jumpNum).hipdy=hipdy';
    jumpDataL(jumpNum).hipd=hipd';
    jumpDataL(jumpNum).hipavgv=hipavgv';
    jumpDataL(jumpNum).hipavgv_vertical = hipavgv_vertical';
    jumpDataL(jumpNum).angularv=angularv';


    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataL(i).time(1);
        plot(jumpDataL(i).time - startTime, jumpDataL(i).hipavgv, 'o')
        hold on
    end
    title("Left Hip Average Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Hip Velocity')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataL(i).time(1);
        plot(jumpDataL(i).time-startTime, jumpDataL(i).hipavgv_vertical, 'o')
        hold on
    end
    title("Left Hip Average Vertical Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Hip Vertical Velocity')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataL(i).time(1);
        plot(jumpDataL(i).time-startTime, jumpDataL(i).hipavgv, 'o')
        hold on
    end
    title("Left Knee Flexion Angle: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Knee Flexion Angle')
    hold off

    figure
    % For loop to graph all data on top of each other
    for i = 1:jumpNum
        startTime = jumpDataL(i).time(1);
        plot(jumpData(L).time-startTime, jumpDataL(i).angularv, 'o')
        hold on
    end
    title("Left Knee Angular Velocity: " + name + string(jumpNum))
    xlabel('Time')
    ylabel('Knee Angular Velocity')
    hold off


    %Wrtie data to Excel sheet
    sheetName = append(name, string(jumpNum));
    S=[jumpDataS(jumpNum, :)];
    writetable(struct2table(S),'Lsagittal.xlsx', 'Sheet', sheetName);
    winopen Lsagittal.xlsx


    %% Frontal Analysis

    gTruthF %display label properties

    gTruthF.LabelDefinitions %display labels

    labelDataF = gTruthF.LabelData; %store label data
    head(labelDataF) %display label data

    %For loop to only get data that has info
    started = false;
    for i = 1:height(labelDataF)
        %if there is no data
        if isempty(labelDataF(i,:).kneer{1})
            %if this is the first instance of no data, set end time to then
            if started == true
                endTime = labelDataF(i,:).Time;
                break;
            else
                continue;

            end
        %if there is data
        else
            %if this is the first instance of data, set start time to then
            if started == false
                startTime = labelDataF(i,:).Time;
                started = true;
            else
                continue;
            end
        end
    end

    gTruthIntervalF = labelDataF(timerange(startTime,endTime),:);
    % should prob change time range to full range of data
    head(gTruthIntervalF)

    % Setting up struct to store jump data
    jumpDataF(jumpNum) = struct('time', [], 'hipx_r', [], 'hipx_l', [], 'kneex_r', [], 'kneex_l', [], ...
    'anklex_r', [], 'anklex_l' , [], 'valgus', []);

    % Boolean to keep track of if there is valgus positioning in both legs
    isValgus = false;

    j = height(gTruthIntervalF); %value for indexing
    x=1;

    % Boolean to keep track of if there are any more hip values or not
    hipr_stopped = false;
    hipl_stopped = false;

    % Setting up arrays to store values
    hipx_r = [];
    hipx_l = [];
    valgus = [];

    while x<=j
        
        %Checking to see if there is marker data for the hips since
        %sometimes the markers do not track the full landing
        if isempty(gTruthIntervalF(x,:).hipr{1})
            if hipr_stopped == false
                last_hipxr = hipx_r(x-1);
            else
            end
            hipx_r(x) = last_hipxr;
        else
            hipx_r(x)= gTruthIntervalF(x,:).hipr{1}(1);
        end

        % Checking left hip and setting value
        if isempty(gTruthIntervalF(x,:).hipl{1})
            if hipl_stopped == false
                last_hipxl = hipx_l(x-1);
            else
            end
            hipx_l(x) = last_hipxl;
        else
            hipx_l(x)= gTruthIntervalF(x,:).hipl{1}(1);
        end


        hipx_r(x)= gTruthIntervalF(x,:).hipr{1}(1);
        hipx_l(x)= gTruthIntervalF(x,:).hipl{1}(1);
        kneex_r(x)= gTruthIntervalF(x,:).kneer{1}(1);
        kneex_l(x)= gTruthIntervalF(x,:).kneel{1}(1);
        anklex_r(x)= gTruthIntervalF(x,:).ankler{1}(1);
        anklex_l(x)= gTruthIntervalF(x,:).anklel{1}(1);
        time(x) = datenum((gTruthIntervalF(x,:).Time)*86400);

        %Checking if there is valgus positioning on right
        if kneex_r(x)<hipx_r(x) && kneex_r(x)<anklex_r(x)
            valgus_r(x) = "Valgus";
        else
            valgus_r(x) = "No Valgus";
        end

        %Checking if there is valgus positioning on left
        if kneex_l(x)>hipx_l(x) && kneex_l(x)>anklex_l(x)
            valgus_l(x) = "Valgus";
        else
            valgus_l(x) = "No Valgus";
        end

        if strcmp(valgus_r(x),"Valgus") && strcmp(valgus_l, "Valgus")
            isValgus = true;
            valgus(x) = "Valgus";
        else
            valgus(x) = "";
        end

        x=x+1;
    end

    if isValgus == true
        fprintf('Valgus Positioning Detected \n')
    else
    end

    jumpDataF(jumpNum).time = time';
    jumpDataF(jumpNum).hipx_r = hipx_r';
    jumpDataF(jumpNum).hipx_l = hipx_l';
    jumpDataF(jumpNum).kneex_r = kneex_r';
    jumpDataF(jumpNum).kneex_l = kneex_l';
    jumpDataF(jumpNum).anklex_r = anklex_r';
    jumpDataF(jumpNum).anklex_l = anklex_l';
    jumpDataF(jumpNum).valgus = valgus';


    %Write frontal data to spreadsheet
    sheetName = append(name, string(jumpNum));
    S=[jumpDataF(jumpNum, :)];
    writetable(struct2table(S),'Frontal.xlsx', 'Sheet', sheetName);
    winopen Frontal.xlsx

    %% Updating and continuing

    % Need to update variables in map
    tempMap = map(name);
    tempMap.jumpDataR = jumpDataR;
    tempMap.jumpDataL = jumpDataL;
    tempMap.jumpDataF = jumpDataF;
    map(name) = tempMap;
    
    while 1
    isDone = input('Add another video? (Y/N)', 's');
    if (isDone == 'Y')
        done = false;
        break
    elseif (isDone == 'N')
            done = true;
            break
    else
        fprintf('Invalid response. Please write Y or N \n');
        continue;
    end
    end
end
